import {GridStack} from "gridstack";
import css from "gridstack/dist/gridstack.min.css";
import style from "./style.css";
import modal from "./modal.css";


window.Vaadin.Flow.stackConnector = {
    initLazy: function (widget) {
        getToken();

        let parse = JSON.parse(widget);


        let items = [];
        for (let i = 0; i < parse.length; i++) {
            var parseElement = parse[i];
            items[i] = {
                content: parseElement.content, w: parseElement.width, h: parseElement.height,
                x: parseElement.x, y: parseElement.y, id: parseElement.id
            };
        }

        let grid = GridStack.init({maxRow: 10, minRow: 5});
        grid.load(items);

        let elements = document.getElementsByClassName("grid-stack-item");
        for (let i = 0; i < elements.length; i++) {
            let element = elements[i];
            let id = element.getAttribute("gs-id");
            element.setAttribute("id", "g" + id);

            document.getElementById(id).addEventListener('click', () => removeWidget(grid, id));
        }

        document.getElementById("btn").addEventListener('click', () => {
            addWidget(grid);
        });

        document.getElementById("saveBtn").addEventListener('click', save);

        document.getElementById("cancel").addEventListener('click', cancelSelect);

        getTemplates();

        enterListener();

    }
}

function enterListener(){
    document.addEventListener('keydown', function(event) {
        if (event.key === 'Enter') {
            let btn = document.getElementById("btn");
            if (btn.style.display === 'block'){
                btn.click();
            }
        }
    });
}

function getTemplates(){
    let token = sessionStorage.getItem("accessToken");

    if (token === undefined || token == null){ token = getToken()}

    fetch('http://localhost:8080/rest/component/templates/all', {
        headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + token
        },
    }).then(response => response.json())
        .then(json => {
            for (let i = 0; i < json.length; i++) {
                let e = json[i];

                document.getElementById("widgets").insertAdjacentHTML('beforeend',
                    ` <div class="widget" id="${e.id}">${e.content}</div>`);

                let elementById = document.getElementById(e.id);
                elementById.addEventListener('click', ()=>myFunction(elementById));
            }
        });
}

function addWidget(grid){
    let token = sessionStorage.getItem("accessToken");

    if (token === undefined || token == null){ token = getToken()}

    fetch('http://localhost:8080/rest/component/create', {
        headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + token
        }
    })
        .then(response => response.json())
        .then(data => {
            let id = data;

            let element1 = document.querySelector(".click");

            element1.insertAdjacentHTML('beforeend', `<button class="exit" id="${id}">X</button>`);

            let content = element1.innerHTML;

            element1.classList.remove("click");

            document.getElementById("btn").style.display = 'none';
            document.getElementById("cancel").style.display = 'none';

            let item = {content: content, id: id, w: 2, h: 2};
            if (grid.willItFit(item)) {

                grid.addWidget(item);

                let classes = document.getElementsByClassName("grid-stack-item");
                let element = classes.item(classes.length - 1);

                // let id = element.getAttribute("gs-id");
                element.setAttribute("id", "g" + id);
                document.getElementById(id).addEventListener('click', () => removeWidget(grid, id));

                save();
            } else {
                alert('Not enough free space to place the widget');
            }
        })
        .catch(error => {
            console.error("Xatolik yuz berdi:", error);
        });
}

function removeWidget(grid, id) {
    grid.removeWidget("#g" + id);

    let token = sessionStorage.getItem("accessToken");

    if (token === undefined || token == null) token = getToken();

    fetch('http://localhost:8080/rest/component/remove', {
        method: 'DELETE',
        headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + token
        },
        body: JSON.stringify(id)
    })
        .then(response => {
            console.log(response);
        })
        .catch(error => {
            console.error("Xatolik yuz berdi:", error);
        });
}

function saveWidgets(widgets, accessToken) {
    fetch('http://localhost:8080/rest/component/change', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/json',
            'Authorization': 'Bearer ' + accessToken
        },
        body: JSON.stringify(widgets)
    })
        .then(response => {
            // console.log(response);
        })
        .catch(error => {
            console.error("Xatolik yuz berdi:", error);
        });
}

function save() {
    let elements = document.getElementsByClassName("grid-stack-item");
    let contents = document.getElementsByClassName("grid-stack-item-content");

    let widgets = new Array(elements.length);

    for (let i = 0; i < elements.length; i++) {
        let element = elements[i];

        let w = element.getAttribute('gs-w');
        w = (w != null ? w : "1");
        let h = element.getAttribute('gs-h');
        h = (h != null ? h : "1");
        let x = element.getAttribute('gs-x');
        let y = element.getAttribute('gs-y');
        let id = element.getAttribute('gs-id');

        widgets[i] = {
            content: contents[i].innerHTML, width: w, height: h,
            x: x, y: y, id: id
        };
    }

    let token = sessionStorage.getItem("accessToken");

    if (token === undefined || token == null) token = getToken();

    saveWidgets(widgets, token);

}

function getToken() {
    let accessToken;

    fetch('http://localhost:8080/oauth/token', {
        method: 'POST',
        headers: {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Authorization': 'Basic ' + btoa('client:secret')
        },
        body: 'grant_type=password&username=admin&password=admin'
    })
        .then(response => response.json())
        .then(data => {
            accessToken = data.access_token;

            sessionStorage.setItem("accessToken", accessToken);
        })
        .catch(error => console.error('Xatolik yuz berdi:', error));

    return accessToken;

}

function cancelSelect(){
    document.querySelector(".click").classList.remove("click");

    document.getElementById("btn").style.display = 'none';
    document.getElementById("cancel").style.display = 'none';
}

function myFunction(e){
    if (e.classList.contains("click")){
        e.classList.remove("click");

        document.getElementById("btn").style.display = 'none';
        document.getElementById("cancel").style.display = 'none';
    } else {
        let elementsByClassName = document.getElementsByClassName("click");
        for (let j = 0; j < elementsByClassName.length; j++) {
            elementsByClassName[j].classList.remove("click");
        }

        e.classList.add("click");
        document.getElementById("btn").style.display = 'block';
        document.getElementById("cancel").style.display = 'block';
    }
}

